from chat_agent import ChatAgent
import sys
import time
import threading
import itertools
from colorama import init, Fore, Style  # 添加颜色支持
import os
import random

# 初始化颜色支持
init()

def loading_animation():
    """加载动画"""
    # 更丰富的加载动画帧
    frames = [
        "🤔 思考中 ",
        "💭 思考中 ",
        "💡 思考中 ",
        "✨ 思考中 "
    ]
    dots = ["", ".", "..", "..."]
    i = 0
    while loading_animation.running:
        frame = frames[i % len(frames)]
        dot = dots[i % len(dots)]
        sys.stdout.write('\r' + Fore.BLUE + frame + dot + Style.RESET_ALL)
        sys.stdout.flush()
        time.sleep(0.3)
        i += 1

def start_loading():
    loading_animation.running = True
    threading.Thread(target=loading_animation).start()

def stop_loading():
    loading_animation.running = False
    sys.stdout.write('\r' + ' ' * 20 + '\r')  # 清除加载动画

def stream_output(text):
    """流式输出文本"""
    # 添加打字机效果的前缀
    sys.stdout.write(Fore.BLUE + "✏️ " + Style.RESET_ALL)
    for char in text:
        print(char, end='', flush=True)
        # 随机化打字速度，使其更自然
        time.sleep(random.uniform(0.01, 0.03))
    print()

def print_welcome():
    """打印欢迎信息"""
    welcome_text = """
╭──────────────────────────────────────╮
│     欢迎使用 AI 知识库助手           │
│                                      │
│  - 输入问题即可开始对话             │
│  - 输入 'q' 退出程序                │
│  - 输入 'clear' 清屏                │
╰──────────────────────────────────────╯
"""
    print(Fore.GREEN + welcome_text + Style.RESET_ALL)

def animate_generating():
    """生成回答的动画效果"""
    frames = [
        "💫 生成回答中",
        "💫 生成回答中.",
        "💫 生成回答中..",
        "💫 生成回答中..."
    ]
    i = 0
    while animate_generating.running:
        sys.stdout.write('\r' + Fore.BLUE + frames[i % len(frames)] + Style.RESET_ALL)
        sys.stdout.flush()
        time.sleep(0.3)
        i += 1

def start_generating_animation():
    """开始生成回答动画"""
    animate_generating.running = True
    threading.Thread(target=animate_generating).start()

def stop_generating_animation():
    """停止生成回答动画"""
    animate_generating.running = False
    sys.stdout.write('\r' + ' ' * 30 + '\r')  # 清除动画

def main():
    print_welcome()
    
    # 检查数据目录
    data_dir = "data"
    if not os.path.exists(data_dir):
        print(Fore.YELLOW + f"\n创建数据目录: {data_dir}" + Style.RESET_ALL)
        os.makedirs(data_dir)
        print(Fore.YELLOW + f"\n请将PDF文件放入以下目录：" + Style.RESET_ALL)
        print(Fore.CYAN + f"{os.path.abspath(data_dir)}" + Style.RESET_ALL)
        print(Fore.YELLOW + f"然后重新运行程序。" + Style.RESET_ALL)
        return
    
    print(Fore.YELLOW + "正在初始化知识库..." + Style.RESET_ALL)
    start_loading()
    
    try:
        agent = ChatAgent("data")
        stop_loading()
        
        if not os.listdir(data_dir):  # 检查目录是否为空
            print(Fore.YELLOW + f"\n提示：{data_dir} 目录为空" + Style.RESET_ALL)
            print(Fore.YELLOW + f"请将PDF文件放入以下目录：" + Style.RESET_ALL)
            print(Fore.CYAN + f"{os.path.abspath(data_dir)}" + Style.RESET_ALL)
            return
            
        print(Fore.GREEN + "\n✓ 知识库加载完成！" + Style.RESET_ALL)
        print(Fore.CYAN + "\n请输入您的问题：" + Style.RESET_ALL)
        
        while True:
            # 用户输入提示
            sys.stdout.write(Fore.GREEN + "\n你: " + Style.RESET_ALL)
            query = input()
            
            if query.lower() == 'q':
                break
            elif query.lower() == 'clear':
                # 清屏
                os.system('cls' if os.name == 'nt' else 'clear')
                print_welcome()
                continue
            elif not query.strip():
                continue
            
            # 显示思考动画
            start_loading()
            
            try:
                # 停止思考动画
                stop_loading()
                # 开始生成回答动画（移除第一个静态提示）
                start_generating_animation()
                sys.stdout.write(Fore.BLUE + "\nAI: " + Style.RESET_ALL)
                
                try:
                    for chunk in agent.chat(query):
                        # 停止生成回答动画并清除动画文本
                        stop_generating_animation()
                        if isinstance(chunk, str):
                            stream_output(chunk)
                        elif isinstance(chunk, dict) and 'answer' in chunk:
                            stream_output(chunk['answer'])
                            
                except Exception as e:
                    stop_generating_animation()
                    print(Fore.RED + f"\n发生错误: {str(e)}" + Style.RESET_ALL)
                    
            except Exception as e:
                stop_loading()
                stop_generating_animation()  # 确保动画停止
                print(Fore.RED + f"\n发生错误: {str(e)}" + Style.RESET_ALL)
                
        print(Fore.YELLOW + "\n感谢使用！再见！" + Style.RESET_ALL)
                
    except Exception as e:
        stop_loading()
        stop_generating_animation()  # 确保动画停止
        print(Fore.RED + f"\n初始化失败: {str(e)}" + Style.RESET_ALL)
        return

if __name__ == "__main__":
    # 检查是否在 Windows 终端中运行
    if os.name == 'nt':
        os.system('color')
    main() 